#!/bin/perl
# to emulate "bash -x script" behavior use "perl -d script" plus commands t,r,q
use strict;
use warnings;
use POSIX 'strftime';
use constant DEBUG => 0; # logging 0=off, 1=on
my $file = shift or die "Usage: $0 pcap-file\n";
die "Unable to read file '$file' ($!)" unless -r $file;

my ($start, $end);
my %host;
my $tcpnr = -1; # to detect new TCP stream
sub new_tcp_stream()
{
	if ($tcpnr >= 0) {
		printf "\nStream %d, Start %s.%03d, End %s.%03d, Duration %.4f sec\n", $tcpnr,
			strftime("%T", localtime($start)), ($start - int($start)) * 1000,
			strftime("%T", localtime($end)), ($end - int($end)) * 1000,
			$end - $start;
		foreach my $detail(sort keys %host)
			{ printf "%3d packets from %s\n", $host{$detail}, $detail }
	}
	# reset for next stream
	%host = ();	
}

my $command = "tshark -r '$file' -n -T fields"
	. " -e tcp.stream -e frame.time_epoch"
	. " -e _ws.col.Source -e _ws.col.Info -Y tcp"
	. "| sort -k 1,1n -k 2,2n"; # sort by column 1 & 2 numeric

open(my $fh, "-|", $command) or die "Command failed $!";
while (<$fh>) {
	print STDERR "$_\n" if DEBUG;
	my ($tcp, $time, $ipsrc, $info) = split "\t";
	if ($tcp != $tcpnr) { 
		new_tcp_stream();
		$tcpnr = $tcp; $start = $time
	}
	# process fields here…
	$end = $time;
	$host{$ipsrc}++	
}
close($fh);
new_tcp_stream() if $tcpnr >= 0;
